@@ -27,6 +27,7 @@
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include "bacon-volume.h"
 
 #define SCALE_SIZE 100
@@ -45,9 +46,19 @@ static gboolean	bacon_volume_button_scroll	(GtkWidget      * widget,
 						 GdkEventScroll * event);
 static gboolean	bacon_volume_button_press	(GtkWidget      * widget,
 						 GdkEventButton * event);
-static gboolean cb_dock_press			(GtkWidget      * widget,
+static gboolean bacon_volume_key_release	(GtkWidget      * widget,
+    						 GdkEventKey    * event);
+static void bacon_volume_button_style_set	(GtkWidget *widget,
+						 GtkStyle *previous_style);
+static gboolean cb_dock_button_press		(GtkWidget      * widget,
 						 GdkEventButton * event,
 						 gpointer         data);
+static gboolean cb_dock_key_release		(GtkWidget      * widget,
+						 GdkEventKey    * event,
+						 gpointer         data);
+static gboolean cb_dock_key_press		(GtkWidget      * widget,
+						 GdkEventKey    * event,
+						 gpointer         data);
 
 static gboolean cb_button_press			(GtkWidget      * widget,
 						 GdkEventButton * event,
@@ -55,7 +66,9 @@ static gboolean cb_button_press			(GtkWidget      * widget,
 static gboolean cb_button_release		(GtkWidget      * widget,
 						 GdkEventButton * event,
 						 gpointer         data);
+static void     bacon_volume_button_update_icon (BaconVolumeButton *button);
 static void	bacon_volume_scale_value_changed(GtkRange       * range);
+static void	bacon_volume_button_load_icons	(GtkWidget      * widget);
 
 /* see below for scale definitions */
 static GtkWidget *bacon_volume_scale_new	(BaconVolumeButton * button,
@@ -70,8 +83,8 @@ bacon_volume_button_get_type (void)
 {
   static GType bacon_volume_button_type = 0;
 
-  if (!bacon_volume_button_type) {
-    static const GTypeInfo bacon_volume_button_info = {
+  if (G_UNLIKELY (bacon_volume_button_type == 0)) {
+    const GTypeInfo bacon_volume_button_info = {
       sizeof (BaconVolumeButtonClass),
       NULL,
       NULL,
@@ -99,12 +112,14 @@ bacon_volume_button_class_init (BaconVolumeButtonClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS (klass);
 
-  parent_class = g_type_class_ref (GTK_TYPE_BUTTON);
+  parent_class = g_type_class_peek_parent (klass);
 
   /* events */
   gobject_class->dispose = bacon_volume_button_dispose;
   gtkwidget_class->button_press_event = bacon_volume_button_press;
+  gtkwidget_class->key_release_event = bacon_volume_key_release;
   gtkwidget_class->scroll_event = bacon_volume_button_scroll;
+  gtkwidget_class->style_set = bacon_volume_button_style_set;
 
   /* signals */
   signals[SIGNAL_VALUE_CHANGED] = g_signal_new ("value-changed",
@@ -119,30 +134,29 @@ bacon_volume_button_init (BaconVolumeButton *button)
   button->timeout = FALSE;
   button->click_id = 0;
   button->dock = button->scale = NULL;
-#ifndef HAVE_GTK_ONLY
-  button->theme = gtk_icon_theme_get_default ();
-#endif
 }
 
 static void
 bacon_volume_button_dispose (GObject *object)
 {
   BaconVolumeButton *button = BACON_VOLUME_BUTTON (object);
+  guint i;
 
   if (button->dock) {
     gtk_widget_destroy (button->dock);
     button->dock = NULL;
   }
 
-  if (button->theme) {
-    g_object_unref (G_OBJECT (button->theme));
-    button->theme = NULL;
-  }
-
   if (button->click_id != 0) {
     g_source_remove (button->click_id);
     button->click_id = 0;
   }
+  for (i = 0; i < 4; i++) {
+    if (button->icon[i] != NULL) {
+      g_object_unref (button->icon[i]);
+      button->icon[i] = NULL;
+    }
+  }
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -160,20 +174,25 @@ bacon_volume_button_new (GtkIconSize size,
   GtkWidget *frame, *box;
 
   button = g_object_new (BACON_TYPE_VOLUME_BUTTON, NULL);
+  atk_object_set_name (gtk_widget_get_accessible (GTK_WIDGET (button)),
+		       _("Volume"));
   button->size = size;
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+  gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
 
-#ifndef HAVE_GTK_ONLY
   /* image */
   button->image = gtk_image_new ();
   gtk_container_add (GTK_CONTAINER (button), button->image);
   gtk_widget_show_all (button->image);
-#endif
 
   /* window */
   button->dock = gtk_window_new (GTK_WINDOW_POPUP);
   g_signal_connect (button->dock, "button-press-event",
-		    G_CALLBACK (cb_dock_press), button);
+		    G_CALLBACK (cb_dock_button_press), button);
+  g_signal_connect (button->dock, "key-release-event",
+		    G_CALLBACK (cb_dock_key_release), button);
+  g_signal_connect (button->dock, "key-press-event",
+		    G_CALLBACK (cb_dock_key_press), button);
   gtk_window_set_decorated (GTK_WINDOW (button->dock), FALSE);
 
   /* frame */
@@ -185,6 +204,8 @@ bacon_volume_button_new (GtkIconSize size,
 
   /* + */
   button->plus = gtk_button_new_with_label (_("+"));
+  atk_object_set_name (gtk_widget_get_accessible (button->plus),
+		       _("Volume Down"));
   gtk_button_set_relief (GTK_BUTTON (button->plus), GTK_RELIEF_NONE);
   g_signal_connect (button->plus, "button-press-event",
 		    G_CALLBACK (cb_button_press), button);
@@ -201,6 +222,8 @@ bacon_volume_button_new (GtkIconSize size,
 
   /* - */
   button->min = gtk_button_new_with_label (_("-"));
+  atk_object_set_name (gtk_widget_get_accessible (button->min),
+		       _("Volume Up"));
   gtk_button_set_relief (GTK_BUTTON (button->min), GTK_RELIEF_NONE);
   g_signal_connect (button->min, "button-press-event",
 		   G_CALLBACK (cb_button_press), button);
@@ -208,8 +231,8 @@ bacon_volume_button_new (GtkIconSize size,
 		    G_CALLBACK (cb_button_release), button);
   gtk_box_pack_start (GTK_BOX (box), button->min, TRUE, FALSE, 0);
 
-  /* call callback once so original icon is drawn */
-  bacon_volume_scale_value_changed (GTK_RANGE (button->scale));
+  /* set button text */
+  bacon_volume_button_update_icon (button);
 
   return GTK_WIDGET (button);
 }
@@ -235,6 +258,15 @@ bacon_volume_button_set_value (BaconVolumeButton * button,
  * button callbacks.
  */
 
+static void
+bacon_volume_button_style_set (GtkWidget *widget,
+			       GtkStyle *previous_style)
+{
+  GTK_WIDGET_CLASS (parent_class)->style_set (widget, previous_style);
+
+  bacon_volume_button_load_icons (widget);
+}
+
 static gboolean
 bacon_volume_button_scroll (GtkWidget      * widget,
 			    GdkEventScroll * event)
@@ -270,8 +302,15 @@ bacon_volume_button_press (GtkWidget      * widget,
   gint x, y, m, dx, dy, sx, sy, ystartoff, mouse_y;
   float v;
   GdkEventButton *e;
+  GdkDisplay *display;
+  GdkScreen *screen;
+
+  display = gtk_widget_get_display (widget);
+  screen = gtk_widget_get_screen (widget);
 
   /* position roughly */
+  gtk_window_set_screen (GTK_WINDOW (button->dock), screen);
+
   gdk_window_get_origin (widget->window, &x, &y);
   x += widget->allocation.x;
   y += widget->allocation.y;
@@ -300,12 +339,25 @@ bacon_volume_button_press (GtkWidget      * widget,
   GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
 
   /* grab focus */
-  gtk_widget_grab_focus (button->dock);
   gtk_grab_add (button->dock);
-  gdk_pointer_grab (button->dock->window, TRUE,
-      GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-      GDK_POINTER_MOTION_MASK, NULL, NULL, GDK_CURRENT_TIME);
-  gdk_keyboard_grab (button->dock->window, TRUE, GDK_CURRENT_TIME);
+
+  if (gdk_pointer_grab (button->dock->window, TRUE,
+			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+			GDK_POINTER_MOTION_MASK, NULL, NULL, event->time)
+      != GDK_GRAB_SUCCESS) {
+	  gtk_grab_remove (button->dock);
+	  gtk_widget_hide (button->dock);
+	  return FALSE;
+   }
+
+  if (gdk_keyboard_grab (button->dock->window, TRUE, event->time) != GDK_GRAB_SUCCESS) {
+	  gdk_display_pointer_ungrab (display, event->time);
+	  gtk_grab_remove (button->dock);
+	  gtk_widget_hide (button->dock);
+	  return FALSE;
+  }
+
+  gtk_widget_grab_focus (button->dock);
 
   /* forward event to the slider */
   e = (GdkEventButton *) gdk_event_copy ((GdkEvent *) event);
@@ -328,6 +380,75 @@ bacon_volume_button_press (GtkWidget      * widget,
   return TRUE;
 }
 
+static gboolean
+bacon_volume_key_release (GtkWidget      * widget,
+    			  GdkEventKey    * event)
+{
+  BaconVolumeButton *button = BACON_VOLUME_BUTTON (widget);
+  GtkAdjustment *adj = gtk_range_get_adjustment (GTK_RANGE (button->scale));
+  gint x, y, m, dx, dy, sx, sy, ystartoff;
+  float v;
+  GdkDisplay *display;
+  GdkScreen *screen;
+
+  if (event->keyval != GDK_space && event->keyval != GDK_Return)
+    return FALSE;
+
+  display = gtk_widget_get_display (widget);
+  screen = gtk_widget_get_screen (widget);
+
+  /* position roughly */
+  gtk_window_set_screen (GTK_WINDOW (button->dock), screen);
+
+  gdk_window_get_origin (widget->window, &x, &y);
+  x += widget->allocation.x;
+  y += widget->allocation.y;
+  gtk_window_move (GTK_WINDOW (button->dock), x, y - (SCALE_SIZE / 2));
+  gtk_widget_show_all (button->dock);
+  gdk_window_get_origin (button->dock->window, &dx, &dy);
+  dy += button->dock->allocation.y;
+  gdk_window_get_origin (button->scale->window, &sx, &sy);
+  sy += button->scale->allocation.y;
+  ystartoff = sy - dy;
+  button->timeout = TRUE;
+
+  /* position (needs widget to be shown already) */
+  v = bacon_volume_button_get_value (button) / (adj->upper - adj->lower);
+  x += (widget->allocation.width - button->dock->allocation.width) / 2;
+  y -= ystartoff;
+  y -= GTK_RANGE (button->scale)->min_slider_size / 2;
+  m = button->scale->allocation.height -
+      GTK_RANGE (button->scale)->min_slider_size;
+  y -= m * (1.0 - v);
+  gtk_window_move (GTK_WINDOW (button->dock), x, y);
+  gdk_window_get_origin (button->scale->window, &sx, &sy);
+
+  /* grab focus */
+  gtk_grab_add (button->dock);
+
+  if (gdk_pointer_grab (button->dock->window, TRUE,
+			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK,
+			NULL, NULL, event->time)
+      != GDK_GRAB_SUCCESS) {
+	  gtk_grab_remove (button->dock);
+	  gtk_widget_hide (button->dock);
+	  return FALSE;
+   }
+
+  if (gdk_keyboard_grab (button->dock->window, TRUE, event->time) != GDK_GRAB_SUCCESS) {
+	  gdk_display_pointer_ungrab (display, event->time);
+	  gtk_grab_remove (button->dock);
+	  gtk_widget_hide (button->dock);
+	  return FALSE;
+  }
+
+  gtk_widget_grab_focus (button->scale);
+
+  button->pop_time = event->time;
+
+  return TRUE;
+}
+
 /*
  * +/- button callbacks.
  */
@@ -405,10 +526,12 @@ bacon_volume_release_grab (BaconVolumeButton *button,
 			   GdkEventButton * event)
 {
   GdkEventButton *e;
+  GdkDisplay *display;
 
   /* ungrab focus */
-  gdk_keyboard_ungrab (GDK_CURRENT_TIME);
-  gdk_pointer_ungrab (GDK_CURRENT_TIME);
+  display = gtk_widget_get_display (GTK_WIDGET (button));
+  gdk_display_keyboard_ungrab (display, event->time);
+  gdk_display_pointer_ungrab (display, event->time);
   gtk_grab_remove (button->dock);
 
   /* hide again */
@@ -424,9 +547,9 @@ bacon_volume_release_grab (BaconVolumeButton *button,
 }
 
 static gboolean
-cb_dock_press (GtkWidget      * widget,
-	       GdkEventButton * event,
-	       gpointer         data)
+cb_dock_button_press (GtkWidget      * widget,
+		      GdkEventButton * event,
+		      gpointer         data)
 {
   //GtkWidget *ewidget = gtk_get_event_widget ((GdkEvent *) event);
   BaconVolumeButton *button = BACON_VOLUME_BUTTON (data);
@@ -439,6 +562,41 @@ cb_dock_press (GtkWidget      * widget,
   return FALSE;
 }
 
+static gboolean
+cb_dock_key_release (GtkWidget      * widget,
+		     GdkEventKey    * event,
+		     gpointer         data)
+{
+  BaconVolumeButton *button = BACON_VOLUME_BUTTON (data);
+
+  if (event->keyval == GDK_Escape) {
+    GdkDisplay *display;
+
+    /* ungrab focus */
+    display = gtk_widget_get_display (widget);
+    gdk_display_keyboard_ungrab (display, event->time);
+    gdk_display_pointer_ungrab (display, event->time);
+
+    /* hide again */
+    gtk_widget_hide (button->dock);
+    button->timeout = FALSE;
+   
+    return TRUE;
+  }
+  return FALSE;
+}
+
+static gboolean
+cb_dock_key_press (GtkWidget      * widget,
+		   GdkEventKey    * event,
+		   gpointer         data)
+{
+  if (event->keyval == GDK_Escape) {
+    return TRUE;
+  }
+  return FALSE;
+}
+
 /*
  * Scale stuff.
  */
@@ -470,8 +628,8 @@ bacon_volume_scale_get_type (void)
 {
   static GType bacon_volume_scale_type = 0;
 
-  if (!bacon_volume_scale_type) {
-    static const GTypeInfo bacon_volume_scale_info = {
+  if (G_UNLIKELY (bacon_volume_scale_type == 0)) {
+    const GTypeInfo bacon_volume_scale_info = {
       sizeof (GtkVScaleClass),
       NULL,
       NULL,
@@ -499,7 +657,7 @@ bacon_volume_scale_class_init (GtkVScaleClass * klass)
   GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS (klass);
   GtkRangeClass *gtkrange_class = GTK_RANGE_CLASS (klass);
 
-  scale_parent_class = g_type_class_ref (GTK_TYPE_VSCALE);
+  scale_parent_class = g_type_class_peek_parent (klass);
 
   gtkwidget_class->button_press_event = bacon_volume_scale_press;
   gtkwidget_class->button_release_event = bacon_volume_scale_release;
@@ -565,42 +723,75 @@ bacon_volume_scale_release (GtkWidget      * widget,
   return res;
 }
 
+static void
+bacon_volume_button_update_icon (BaconVolumeButton *button)
+{
+  GtkRange *range = GTK_RANGE (button->scale);
+  float val = gtk_range_get_value (range);
+  GtkAdjustment *adj;
+  GdkPixbuf *pixbuf;
+  float step;
+
+  adj = gtk_range_get_adjustment (range);
+  step = (adj->upper - adj->lower) / 4;
+
+  if (val == adj->lower)
+    pixbuf = button->icon[0];
+  else if (val > adj->lower && val <= adj->lower + step)
+    pixbuf = button->icon[1];
+  else if (val > adj->lower + step && val <= adj->lower + step * 2)
+    pixbuf = button->icon[2];
+  else
+    pixbuf = button->icon[3];
+
+  gtk_image_set_from_pixbuf (GTK_IMAGE (button->image), pixbuf);
+}
+
 static void
 bacon_volume_scale_value_changed (GtkRange * range)
 {
   BaconVolumeScale *scale = BACON_VOLUME_SCALE (range);
   BaconVolumeButton *button = scale->button;
-  GtkAdjustment *adj = gtk_range_get_adjustment (GTK_RANGE (button->scale));
-  float step = (adj->upper - adj->lower) / 4;
-  float val = gtk_range_get_value (range);
-  gint w, h;
-#ifdef HAVE_GTK_ONLY
-  char *s;
 
-  /* update label */
-  s = g_strdup_printf ("%d", lrintf (val));
-  gtk_button_set_label (GTK_BUTTON (button), s);
-  g_free (s);
-#else
-  const char *s;
-  GdkPixbuf *buf;
-
-  if (val == adj->lower)
-    s = "stock_volume-mute";
-  else if (val > adj->lower && val <= adj->lower + step)
-    s = "stock_volume-min";
-  else if (val > adj->lower + step && val <= adj->lower + step * 2)
-    s = "stock_volume-med";
-  else
-    s = "stock_volume-max";
-
-  /* update image */
-  gtk_icon_size_lookup (button->size, &w, &h);
-  buf = gtk_icon_theme_load_icon (button->theme, s, w, 0, NULL);
-  gtk_image_set_from_pixbuf (GTK_IMAGE (button->image), buf);
-  g_object_unref (buf);
-#endif
+  bacon_volume_button_update_icon (button);
 
   /* signal */
   g_signal_emit (button, signals[SIGNAL_VALUE_CHANGED], 0);
 }
+ 
+static void
+bacon_volume_button_load_icons (GtkWidget *widget)
+{
+  BaconVolumeButton *button = BACON_VOLUME_BUTTON (widget);
+  guint i;
+  gint w, h;
+  GdkScreen *screen;
+  GtkIconTheme *theme;
+  const char *icon_name[] = {"audio-volume-muted", "audio-volume-low",
+    "audio-volume-medium", "audio-volume-high"};
+
+  screen = gtk_widget_get_screen (widget);
+  theme = gtk_icon_theme_get_for_screen (screen);
+
+  gtk_icon_size_lookup (button->size, &w, &h);
+
+  for (i = 0; i < 4; i++) {
+    GError *error = NULL;
+    if (button->icon[i] != NULL) {
+      g_object_unref (button->icon[i]);
+      button->icon[i] = NULL;
+    }
+    button->icon[i] = gtk_icon_theme_load_icon (theme, icon_name[i], w, 0, &error);
+    if (error) {
+      g_print ("Couldn't load themed icon '%s': %s\n", icon_name[i], error->message);
+      g_error_free (error);
+    }
+  }
+
+  /* Apply the new icons */
+  bacon_volume_button_update_icon (button);
+}
+
+/*
+ * vim: sw=2 ts=8 cindent noai bs=2
+ */
