@@ -69,7 +69,6 @@ struct TotemPlaylistPrivate
 
 	/* This is the playing icon */
 	GdkPixbuf *icon;
-	GtkIconTheme *theme;
 
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
@@ -81,14 +80,7 @@ struct TotemPlaylistPrivate
 	char *path;
 	char *save_path;
 
-	/* Repeat mode */
-	gboolean repeat;
-
-	/* Reorder Flag */
-	gboolean drag_started;
-
 	/* Shuffle mode */
-	gboolean shuffle;
 	int *shuffled;
 	int current_shuffled, shuffle_len;
 
@@ -96,7 +88,19 @@ struct TotemPlaylistPrivate
 
 	int x, y;
 
-	gboolean disable_save_to_disk;
+	guint disable_save_to_disk : 1;
+
+	/* Repeat mode */
+	guint repeat : 1;
+
+	/* Reorder Flag */
+	guint drag_started : 1;
+
+	/* Drop disabled flag */
+	guint drop_disabled : 1;
+
+	/* Shuffle mode */
+	guint shuffle : 1;
 };
 
 /* Signals */
@@ -143,8 +147,6 @@ static const GtkActionEntry entries[] = {
 	{ "copy-location", GTK_STOCK_COPY, N_("_Copy location"), NULL, N_("Copy the location to the clipboard"), G_CALLBACK (playlist_copy_location_action_activated) }
 };
 
-static GtkWidgetClass *parent_class = NULL;
-
 static void totem_playlist_class_init (TotemPlaylistClass *class);
 static void totem_playlist_init       (TotemPlaylist      *playlist);
 
@@ -614,10 +616,16 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 static gboolean 
 button_press_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 { 
+	TotemPlaylist *playlist = (TotemPlaylist *)data;
+
+	if (playlist->_priv->drop_disabled)
+		return FALSE;
+
+	playlist->_priv->drop_disabled = TRUE;
 	gtk_drag_dest_unset (treeview);
 	g_signal_handlers_block_by_func (treeview, (GFunc) drop_cb, data);
 
-	totem_playlist_set_reorderable ((TotemPlaylist *) data, TRUE);
+	totem_playlist_set_reorderable (playlist, TRUE);
 
 	return FALSE;
 }
@@ -627,8 +635,9 @@ button_release_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 {
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
 
-	if (!playlist->_priv->drag_started)
+	if (!playlist->_priv->drag_started && playlist->_priv->drop_disabled)
 	{
+		playlist->_priv->drop_disabled = FALSE;
 		totem_playlist_set_reorderable (playlist, FALSE);
 		gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
 				target_table, G_N_ELEMENTS (target_table),
@@ -646,7 +655,7 @@ drag_begin_cb (GtkWidget *treeview, GdkDragContext *context, gpointer data)
 {
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
 
-	playlist->_priv->drag_started= TRUE;
+	playlist->_priv->drag_started = TRUE;
 
 	return;
 }
@@ -656,6 +665,7 @@ drag_end_cb (GtkWidget *treeview, GdkDragContext *context, gpointer data)
 {
 	TotemPlaylist *playlist = (TotemPlaylist *)data;
 
+	playlist->_priv->drop_disabled = FALSE;
 	playlist->_priv->drag_started = FALSE;
 	totem_playlist_set_reorderable (playlist, FALSE);
 	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL, target_table,
@@ -690,23 +700,31 @@ selection_changed (GtkTreeSelection *treeselection, TotemPlaylist *playlist)
 }
 
 static void
-load_icon (TotemPlaylist *playlist)
+totem_playlist_style_set (GtkWidget *widget,
+			  GtkStyle *previous_style)
 {
+	TotemPlaylist *playlist = TOTEM_PLAYLIST (widget);
+	TotemPlaylistPrivate *priv = playlist->_priv;
 	gint w, h;
+	GError *error = NULL;
+	GtkIconTheme *theme;
+	const char icon_name[] = "audio-volume-medium";
 
-	if (playlist->_priv->icon != NULL) {
-		g_object_unref (G_OBJECT (playlist->_priv->icon));
+	GTK_WIDGET_CLASS (totem_playlist_parent_class)->style_set (widget, previous_style);
+
+	if (priv->icon != NULL) {
+		g_object_unref (priv->icon);
 	}
 
-	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
-	playlist->_priv->icon = gtk_icon_theme_load_icon
-		(playlist->_priv->theme, "audio-volume-medium", w, 0, NULL);
-}
+	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
+					   GTK_ICON_SIZE_MENU, &w, &h);
 
-static void
-icon_theme_changed (GtkIconTheme *theme, TotemPlaylist *playlist)
-{
-	load_icon (playlist);
+	theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget));
+	priv->icon = gtk_icon_theme_load_icon (theme, icon_name, w, 0, &error);
+	if (error) {
+		g_print ("Couldn't load themed icon '%s': %s\n", icon_name, error->message);
+		g_error_free (error);
+	}
 }
 
 /* This function checks if the current item is NULL, and try to update it
@@ -1257,8 +1275,16 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 			totem_playlist_table_signals[CHANGED], 0,
 			NULL);
 
-	g_signal_handlers_unblock_by_func (treeview,
-			(GFunc) drop_cb, playlist);
+	if (playlist->_priv->drop_disabled) {
+		playlist->_priv->drop_disabled = FALSE;
+		totem_playlist_set_reorderable (playlist, FALSE);
+		gtk_drag_dest_set (GTK_WIDGET (treeview), GTK_DEST_DEFAULT_ALL,
+				target_table, G_N_ELEMENTS (target_table),
+				GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
+		g_signal_handlers_unblock_by_func (treeview,
+				(GFunc) drop_cb, playlist);
+	}
 }
 
 static void
@@ -1323,7 +1349,7 @@ update_repeat_cb (GConfClient *client, guint cnxn_id,
 	gboolean repeat;
 
 	repeat = gconf_value_get_bool (entry->value);
-	playlist->_priv->repeat = repeat;
+	playlist->_priv->repeat = (repeat != FALSE);
 
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
@@ -1430,7 +1456,7 @@ update_lockdown (GConfClient *client, guint cnxn_id,
 {
 	playlist->_priv->disable_save_to_disk = gconf_client_get_bool
 			(playlist->_priv->gc,
-			"/desktop/gnome/lockdown/disable_save_to_disk", NULL);
+			"/desktop/gnome/lockdown/disable_save_to_disk", NULL) != FALSE;
 	totem_playlist_update_save_button (playlist);
 }
 
@@ -1441,7 +1467,7 @@ init_config (TotemPlaylist *playlist)
 
 	playlist->_priv->disable_save_to_disk = gconf_client_get_bool
 	       		(playlist->_priv->gc,
-			"/desktop/gnome/lockdown/disable_save_to_disk", NULL);
+			"/desktop/gnome/lockdown/disable_save_to_disk", NULL) != FALSE;
 	totem_playlist_update_save_button (playlist);
 
 	gconf_client_add_dir (playlist->_priv->gc, GCONF_PREFIX,
@@ -1461,9 +1487,9 @@ init_config (TotemPlaylist *playlist)
 			playlist, NULL, NULL);
 
 	playlist->_priv->repeat = gconf_client_get_bool (playlist->_priv->gc,
-			GCONF_PREFIX"/repeat", NULL);
+			GCONF_PREFIX"/repeat", NULL) != FALSE;
 	playlist->_priv->shuffle = gconf_client_get_bool (playlist->_priv->gc,
-			GCONF_PREFIX"/shuffle", NULL);
+			GCONF_PREFIX"/shuffle", NULL) != FALSE;
 }
 
 static void
@@ -1474,25 +1500,6 @@ totem_playlist_entry_parsed (TotemPlParser *parser,
 	totem_playlist_add_one_mrl (playlist, uri, title);
 }
 
-static void
-totem_playlist_realize (GtkWidget *widget, TotemPlaylist *playlist)
-{
-	playlist->_priv->theme = gtk_icon_theme_get_for_screen
-		(gtk_widget_get_screen (GTK_WIDGET (widget)));
-	g_signal_connect (G_OBJECT (playlist->_priv->theme), "changed",
-			G_CALLBACK (icon_theme_changed), playlist);
-	load_icon (playlist);
-}
-
-static void
-totem_playlist_unrealize (GtkWidget *widget, TotemPlaylist *playlist)
-{
-	if (playlist->_priv->theme != NULL) {
-		g_object_unref (playlist->_priv->theme);
-		playlist->_priv->theme = NULL;
-	}
-}
-
 static void
 totem_playlist_init (TotemPlaylist *playlist)
 {
@@ -1508,15 +1515,6 @@ totem_playlist_init (TotemPlaylist *playlist)
 			"entry",
 			G_CALLBACK (totem_playlist_entry_parsed),
 			playlist);
-
-	g_signal_connect (G_OBJECT (playlist),
-			"realize",
-			G_CALLBACK (totem_playlist_realize),
-			playlist);
-	g_signal_connect (G_OBJECT (playlist),
-			"unrealize",
-			G_CALLBACK (totem_playlist_unrealize),
-			playlist);
 }
 
 static void
@@ -1542,9 +1540,7 @@ totem_playlist_finalize (GObject *object)
 		g_object_unref (G_OBJECT (playlist->_priv->action_group));
 	}
 
-	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
-		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
-	}
+	G_OBJECT_CLASS (totem_playlist_parent_class)->finalize (object);
 }
 
 GtkWidget*
@@ -1554,7 +1550,7 @@ totem_playlist_new (void)
 	GtkWidget *container, *item;
 	char *filename;
 
-	playlist = TOTEM_PLAYLIST (g_object_new (GTK_TYPE_PLAYLIST, NULL));
+	playlist = TOTEM_PLAYLIST (g_object_new (TOTEM_TYPE_PLAYLIST, NULL));
 
 	playlist->_priv->xml = totem_interface_load ("playlist.glade",
 			_("playlist"), TRUE, NULL);
@@ -1644,7 +1640,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	GtkTreeIter iter;
 	char *filename_for_display, *uri;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 	g_return_val_if_fail (mrl != NULL, FALSE);
 
 	if (display_name == NULL)
@@ -1716,7 +1712,7 @@ totem_playlist_clear (TotemPlaylist *playlist)
 {
 	GtkListStore *store;
 
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	store = GTK_LIST_STORE (playlist->_priv->model);
 	gtk_list_store_clear (store);
@@ -1861,7 +1857,7 @@ char
 	GtkTreeIter iter;
 	char *path;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), NULL);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), NULL);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return NULL;
@@ -1884,7 +1880,7 @@ char
 	GtkTreeIter iter;
 	char *path;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), NULL);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), NULL);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return NULL;
@@ -1910,7 +1906,7 @@ totem_playlist_get_current_metadata (TotemPlaylist *playlist,
 {
 	GtkTreeIter iter;
 
-        g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+        g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
@@ -1935,7 +1931,7 @@ totem_playlist_has_previous_mrl (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
@@ -1964,7 +1960,7 @@ totem_playlist_has_next_mrl (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
@@ -1994,7 +1990,7 @@ totem_playlist_set_title (TotemPlaylist *playlist, const char *title, gboolean f
 	GtkTreeIter iter;
 	gboolean custom_title;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
@@ -2033,7 +2029,7 @@ totem_playlist_set_playing (TotemPlaylist *playlist, gboolean state)
 	GtkTreeIter iter;
 	GtkTreePath *path;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
@@ -2066,7 +2062,7 @@ totem_playlist_set_previous (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
 
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	if (totem_playlist_has_previous_mrl (playlist) == FALSE)
 		return;
@@ -2116,7 +2112,7 @@ totem_playlist_set_next (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
 
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	if (totem_playlist_has_next_mrl (playlist) == FALSE)
 	{
@@ -2152,7 +2148,7 @@ totem_playlist_set_next (TotemPlaylist *playlist)
 gboolean
 totem_playlist_get_repeat (TotemPlaylist *playlist)
 {
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	return playlist->_priv->repeat;
 }
@@ -2160,7 +2156,7 @@ totem_playlist_get_repeat (TotemPlaylist *playlist)
 void
 totem_playlist_set_repeat (TotemPlaylist *playlist, gboolean repeat)
 {
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	gconf_client_set_bool (playlist->_priv->gc, GCONF_PREFIX"/repeat",
 			repeat, NULL);
@@ -2169,7 +2165,7 @@ totem_playlist_set_repeat (TotemPlaylist *playlist, gboolean repeat)
 gboolean
 totem_playlist_get_shuffle (TotemPlaylist *playlist)
 {
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
 	return playlist->_priv->shuffle;
 }
@@ -2177,7 +2173,7 @@ totem_playlist_get_shuffle (TotemPlaylist *playlist)
 void
 totem_playlist_set_shuffle (TotemPlaylist *playlist, gboolean shuffle)
 {
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	gconf_client_set_bool (playlist->_priv->gc, GCONF_PREFIX"/shuffle",
 			shuffle, NULL);
@@ -2186,7 +2182,7 @@ totem_playlist_set_shuffle (TotemPlaylist *playlist, gboolean shuffle)
 void
 totem_playlist_set_at_start (TotemPlaylist *playlist)
 {
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	totem_playlist_unset_playing (playlist);
 
@@ -2203,7 +2199,7 @@ totem_playlist_set_at_end (TotemPlaylist *playlist)
 {
 	int indice;
 
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	totem_playlist_unset_playing (playlist);
 
@@ -2231,7 +2227,7 @@ totem_playlist_get_current (TotemPlaylist *playlist)
 	char *path;
 	double index;
 
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), 0);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), 0);
 
 	path = gtk_tree_path_to_string (playlist->_priv->current);
 	if (path == NULL)
@@ -2246,7 +2242,7 @@ totem_playlist_get_current (TotemPlaylist *playlist)
 guint
 totem_playlist_get_last (TotemPlaylist *playlist)
 {
-	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), -1);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), -1);
 
 	return PL_LEN - 1;
 }
@@ -2254,7 +2250,7 @@ totem_playlist_get_last (TotemPlaylist *playlist)
 void
 totem_playlist_set_current (TotemPlaylist *playlist, guint index)
 {
-	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
 	if (index >= (guint) PL_LEN)
 		return;
@@ -2268,9 +2264,12 @@ totem_playlist_set_current (TotemPlaylist *playlist, guint index)
 static void
 totem_playlist_class_init (TotemPlaylistClass *klass)
 {
-	parent_class = gtk_type_class (gtk_dialog_get_type ());
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	G_OBJECT_CLASS (klass)->finalize = totem_playlist_finalize;
+	object_class->finalize = totem_playlist_finalize;
+
+	widget_class->style_set = totem_playlist_style_set;
 
 	/* Signals */
 	totem_playlist_table_signals[CHANGED] =
