@@ -39,11 +39,10 @@
 #include "totem-private.h"
 #include "totem-preferences.h"
 #include "video-utils.h"
+#include "totem-subtitle-encoding.h"
 
 #include "debug.h"
 
-#define PROPRIETARY_PLUGINS ".gnome2"G_DIR_SEPARATOR_S"totem-addons"
-
 static void
 totem_action_info (char *reason, Totem *totem)
 {
@@ -185,33 +184,24 @@ on_tvout_toggled (GtkToggleButton *togglebutton, Totem *totem)
 		totem_action_info (_("Switching on or off this type of TV-Out requires a restart to take effect."), totem);
 }
 
-static void
-on_deinterlace1_activate (GtkCheckMenuItem *checkmenuitem, Totem *totem)
-{
-	gboolean value;
-
-	value = gtk_check_menu_item_get_active (checkmenuitem);
-	bacon_video_widget_set_deinterlacing (totem->bvw, value);
-	gconf_client_set_bool (totem->gc, GCONF_PREFIX"/deinterlace",
-			value, NULL);
-}
-
 static void
 deinterlace_changed_cb (GConfClient *client, guint cnxn_id,
 		GConfEntry *entry, Totem *totem)
 {
-	GtkWidget *item;
+	GtkAction *action;
 
-	item = glade_xml_get_widget (totem->xml, "tmw_deinterlace_menu_item");
-	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
-			on_deinterlace1_activate, totem);
+	action = gtk_action_group_get_action (totem->main_action_group,
+			"deinterlace");
 
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+	g_signal_handlers_block_matched (G_OBJECT (action),
+			G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, totem);
+
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
 			gconf_client_get_bool (totem->gc,
 				GCONF_PREFIX"/deinterlace", NULL));
 
-	g_signal_connect (G_OBJECT (item),  "activate",
-			G_CALLBACK (on_deinterlace1_activate), totem);
+	g_signal_handlers_unblock_matched (G_OBJECT (action),
+			G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, totem);
 }
 
 static void
@@ -395,6 +385,18 @@ on_font_set (GtkFontButton * fb, Totem * totem)
 				 font, NULL);
 }
 
+static void
+on_encoding_set (GtkComboBox *cb, Totem *totem)
+{
+	const gchar *encoding;
+
+	encoding = totem_subtitle_encoding_get_selected (cb);
+	if (encoding)
+		gconf_client_set_string (totem->gc,
+				GCONF_PREFIX"/subtitle_encoding",
+				encoding, NULL);
+}
+
 static void
 font_changed_cb (GConfClient *client, guint cnxn_id,
 		 GConfEntry *entry, Totem *totem)
@@ -408,15 +410,30 @@ font_changed_cb (GConfClient *client, guint cnxn_id,
 	bacon_video_widget_set_subtitle_font (totem->bvw, font);
 }
 
+static void
+encoding_changed_cb (GConfClient *client, guint cnxn_id,
+		 GConfEntry *entry, Totem *totem)
+{
+	const gchar *encoding;
+	GtkWidget *item;
+
+	item = glade_xml_get_widget (totem->xml, "subtitle_encoding_combo");
+	encoding = gconf_value_get_string (entry->value);
+	totem_subtitle_encoding_set (GTK_COMBO_BOX(item), encoding);
+	bacon_video_widget_set_subtitle_encoding (totem->bvw, encoding);
+}
+
 void
 totem_setup_preferences (Totem *totem)
 {
 	GtkWidget *item, *menu;
+	GtkAction *action;
 	gboolean show_visuals, auto_resize, is_local, deinterlace;
 	int connection_speed, i;
-	char *path, *visual, *font;
+	char *visual, *font, *encoding;
 	GList *list, *l;
 	BaconVideoWidgetAudioOutType audio_out;
+	GConfValue *value;
 
 	g_return_if_fail (totem->gc != NULL);
 
@@ -447,6 +464,9 @@ totem_setup_preferences (Totem *totem)
 			GTK_STOCK_CLOSE,
 			GTK_RESPONSE_ACCEPT,
 			NULL);
+	gtk_dialog_set_has_separator (GTK_DIALOG (totem->prefs), FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (totem->prefs), 5);
+	gtk_box_set_spacing (GTK_BOX(GTK_DIALOG(totem->prefs)->vbox), 2);
 	gtk_widget_reparent (item, GTK_DIALOG (totem->prefs)->vbox);
 	gtk_widget_show_all (GTK_DIALOG (totem->prefs)->vbox);
 	item = glade_xml_get_widget (totem->xml, "totem_preferences_window");
@@ -474,14 +494,6 @@ totem_setup_preferences (Totem *totem)
 	g_signal_connect (item, "changed",
 			G_CALLBACK (connection_combobox_changed), totem);
 
-	/* Proprietary plugins */
-	path = g_build_path (G_DIR_SEPARATOR_S,
-			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
-	if (g_file_test (path, G_FILE_TEST_IS_DIR) == FALSE)
-		mkdir (path, 0775);
-	bacon_video_widget_set_proprietary_plugins_path (totem->bvw, path);
-	g_free (path);
-
 	/* Enable visuals */
 	item = glade_xml_get_widget (totem->xml, "tpw_visuals_checkbutton");
 	show_visuals = gconf_client_get_bool (totem->gc,
@@ -580,33 +592,33 @@ totem_setup_preferences (Totem *totem)
 
 	/* This one is for the deinterlacing menu, not really our dialog
 	 * but we do it anyway */
-	item = glade_xml_get_widget (totem->xml, "tmw_deinterlace_menu_item");
+	action = gtk_action_group_get_action (totem->main_action_group,
+			"deinterlace");
 	deinterlace = gconf_client_get_bool (totem->gc,
 			GCONF_PREFIX"/deinterlace", NULL);
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
-				deinterlace);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+			deinterlace);
 	bacon_video_widget_set_deinterlacing (totem->bvw, deinterlace);
 	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/deinterlace",
 			(GConfClientNotifyFunc) deinterlace_changed_cb,
 			totem, NULL, NULL);
-	g_signal_connect (G_OBJECT (item), "activate",
-			G_CALLBACK (on_deinterlace1_activate), totem);
 
 	/* Always On Top */
-	item = glade_xml_get_widget (totem->xml, "tmw_always_on_top_menu_item");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+	action = gtk_action_group_get_action (totem->main_action_group,
+			"always-on-top");
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
 			gconf_client_get_bool (totem->gc,
 				GCONF_PREFIX"/window_on_top", NULL));
 
 	/* Save to disk Lockdown */
-	item = glade_xml_get_widget
-		(totem->xml, "tmw_take_screenshot_menu_item");
-	gtk_widget_set_sensitive (item,
+	action = gtk_action_group_get_action (totem->main_action_group,
+			"take-screenshot");
+	gtk_action_set_sensitive (action,
 			!gconf_client_get_bool (totem->gc,
 				"/desktop/gnome/lockdown/disable_save_to_disk",
 				NULL));
 
-	/* subtitle font selection */
+	/* Subtitle font selection */
 	item = glade_xml_get_widget (totem->xml, "font_sel_button");
 	g_signal_connect (item, "font-set", G_CALLBACK (on_font_set), totem);
 	gtk_font_button_set_title (GTK_FONT_BUTTON (item),
@@ -621,6 +633,37 @@ totem_setup_preferences (Totem *totem)
 	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/subtitle_font",
 			(GConfClientNotifyFunc) font_changed_cb,
 			totem, NULL, NULL);
+
+	/* Subtitle encoding selection */
+	item = glade_xml_get_widget (totem->xml, "subtitle_encoding_combo");
+	totem_subtitle_encoding_init (GTK_COMBO_BOX (item));
+	g_signal_connect (item, "changed", G_CALLBACK (on_encoding_set), totem);
+	value = gconf_client_get_without_default (totem->gc,
+			GCONF_PREFIX"/subtitle_encoding", NULL);
+	/* Make sure the default is UTF-8 */
+	if (value != NULL) {
+		if (gconf_value_get_string (value) == NULL) {
+			encoding = g_strdup ("UTF-8");
+		} else {
+			encoding = g_strdup (gconf_value_get_string (value));
+			if (encoding[0] == '\0') {
+				g_free (encoding);
+				encoding = g_strdup ("UTF-8");
+			}
+		}
+		gconf_value_free (value);
+	} else {
+		encoding = g_strdup ("UTF-8");
+	}
+	totem_subtitle_encoding_set (GTK_COMBO_BOX(item), encoding);
+	if (encoding && strcasecmp (encoding, "") != 0) {
+		bacon_video_widget_set_subtitle_encoding (totem->bvw, encoding);
+	}
+	g_free (encoding);
+	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/subtitle_encoding",
+			(GConfClientNotifyFunc) encoding_changed_cb,
+			totem, NULL, NULL);
+
 }
 
 void
@@ -642,9 +685,6 @@ totem_preferences_tvout_setup (Totem *totem)
 	case TV_OUT_NVTV_NTSC:
 		name = "tpw_nvtvntscmode_radio_button";
 		break;
-	case TV_OUT_DXR3:
-		name = "tpw_dxr3tvout_radio_button";
-		break;
 	default:
 		g_assert_not_reached ();
 		name = NULL;
@@ -676,14 +716,6 @@ totem_preferences_tvout_setup (Totem *totem)
                 GINT_TO_POINTER (TV_OUT_NVTV_NTSC));
 	gtk_widget_set_sensitive(item, 
 			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NVTV_NTSC));
-	
-	item = glade_xml_get_widget (totem->xml, "tpw_dxr3tvout_radio_button");
-	g_signal_connect (G_OBJECT (item), "toggled",
-			G_CALLBACK (on_tvout_toggled), totem);
-	g_object_set_data (G_OBJECT (item), "tvout_type",
-			GINT_TO_POINTER (TV_OUT_DXR3));
-	gtk_widget_set_sensitive(item, 
-			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_DXR3));
 }
 
 void
